pub enum StatementKind {
Show 19 variants
Assign(Place, Rvalue),
SetDiscriminant(Place, VariantId),
StorageLive(LocalId),
StorageDead(LocalId),
PlaceMention(Place),
Borrowck(BorrowckStatement),
Drop {
place: Place,
fn_ptr: FnPtr,
kind: DropKind,
on_unwind: Block,
},
Assert {
assert: Assert,
on_failure: AbortKind,
on_unwind: Block,
},
InlineAsm {
asm: String,
targets: Vec<Block>,
on_unwind: Block,
},
Call {
call: Call,
on_unwind: Block,
},
Abort(AbortKind),
Return,
UnwindResume,
Break(usize),
Continue(usize),
Nop,
Switch(Switch),
Loop(Block),
Error(String),
}Expand description
A raw statement: a statement without meta data.
Variants
Assign(Place, Rvalue)
Assigns an Rvalue to a Place. e.g. let y = x; could become
y := move x which is represented as Assign(y, Rvalue::Use(Operand::Move(x))).
SetDiscriminant(Place, VariantId)
Not used today because we take MIR built.
StorageLive(LocalId)
Indicates that this local should be allocated; if it is already allocated, this frees
the local and re-allocates it. The arguments do not receive a StorageLive. We ensure in
the micro-pass insert_storage_statements that all other locals have a StorageLive
associated with them.
StorageDead(LocalId)
Indicates that this local should be deallocated; if it is already deallocated, this is
a no-op. A local may not have a StorageDead in the function’s body, in which case it
is implicitly deallocated at the end of the function. The return local does not receive a
StorageDead. We ensure in the micro-pass insert_storage_statements that all other locals
have a StorageDead before function exits.
PlaceMention(Place)
A place is mentioned, but not accessed. The place itself must still be valid though, so this statement is not a no-op: it can trigger UB if the place’s projections are not valid (e.g. because they go out of bounds).
Borrowck(BorrowckStatement)
Statements that only affect borrow-checking.
Drop
Drop the value at the given place.
Depending on DropKind, this may be a real call to drop_glue, or a conditional call
that should only happen if the place has not been moved out of. See the docs of DropKind
for more details; to get precise drops use --precise-drops.
Fields
Assert
InlineAsm
An inline assembly block. For now we only preserve the template string.
Call
Abort(AbortKind)
Panic also handles “unreachable”. We keep the name of the panicking function that was called.
Return
UnwindResume
Unwind out of the current function into its caller.
Break(usize)
Break to outer loops.
The usize gives the index of the outer loop to break to:
- 0: break to first outer loop (the current loop)
- 1: break to second outer loop
- …
Continue(usize)
Continue to outer loops.
The usize gives the index of the outer loop to continue to:
- 0: continue to first outer loop (the current loop)
- 1: continue to second outer loop
- …
Nop
No-op.
Switch(Switch)
Loop(Block)
Error(String)
Implementations
Sourceimpl StatementKind
impl StatementKind
pub fn is_assign(&self) -> bool
pub fn is_set_discriminant(&self) -> bool
pub fn is_storage_live(&self) -> bool
pub fn is_storage_dead(&self) -> bool
pub fn is_place_mention(&self) -> bool
pub fn is_borrowck(&self) -> bool
pub fn is_drop(&self) -> bool
pub fn is_assert(&self) -> bool
pub fn is_inline_asm(&self) -> bool
pub fn is_call(&self) -> bool
pub fn is_abort(&self) -> bool
pub fn is_return(&self) -> bool
pub fn is_unwind_resume(&self) -> bool
pub fn is_break(&self) -> bool
pub fn is_continue(&self) -> bool
pub fn is_nop(&self) -> bool
pub fn is_switch(&self) -> bool
pub fn is_loop(&self) -> bool
pub fn is_error(&self) -> bool
Sourceimpl StatementKind
impl StatementKind
pub fn to_assign(self) -> Option<(Place, Rvalue)>
pub fn to_set_discriminant(self) -> Option<(Place, VariantId)>
pub fn to_storage_live(self) -> Option<LocalId>
pub fn to_storage_dead(self) -> Option<LocalId>
pub fn to_place_mention(self) -> Option<Place>
pub fn to_borrowck(self) -> Option<BorrowckStatement>
pub fn to_drop(self) -> Option<(Place, FnPtr, DropKind, Block)>
pub fn to_assert(self) -> Option<(Assert, AbortKind, Block)>
pub fn to_inline_asm(self) -> Option<(String, Vec<Block>, Block)>
pub fn to_call(self) -> Option<(Call, Block)>
pub fn to_abort(self) -> Option<AbortKind>
pub fn to_return(self) -> Option<()>
pub fn to_unwind_resume(self) -> Option<()>
pub fn to_break(self) -> Option<usize>
pub fn to_continue(self) -> Option<usize>
pub fn to_nop(self) -> Option<()>
pub fn to_switch(self) -> Option<Switch>
pub fn to_loop(self) -> Option<Block>
pub fn to_error(self) -> Option<String>
Sourceimpl StatementKind
impl StatementKind
pub fn as_assign(&self) -> Option<(&Place, &Rvalue)>
pub fn as_set_discriminant(&self) -> Option<(&Place, &VariantId)>
pub fn as_storage_live(&self) -> Option<&LocalId>
pub fn as_storage_dead(&self) -> Option<&LocalId>
pub fn as_place_mention(&self) -> Option<&Place>
pub fn as_borrowck(&self) -> Option<&BorrowckStatement>
pub fn as_drop(&self) -> Option<(&Place, &FnPtr, &DropKind, &Block)>
pub fn as_assert(&self) -> Option<(&Assert, &AbortKind, &Block)>
pub fn as_inline_asm(&self) -> Option<(&String, &Vec<Block>, &Block)>
pub fn as_call(&self) -> Option<(&Call, &Block)>
pub fn as_abort(&self) -> Option<&AbortKind>
pub fn as_return(&self) -> Option<()>
pub fn as_unwind_resume(&self) -> Option<()>
pub fn as_break(&self) -> Option<&usize>
pub fn as_continue(&self) -> Option<&usize>
pub fn as_nop(&self) -> Option<()>
pub fn as_switch(&self) -> Option<&Switch>
pub fn as_loop(&self) -> Option<&Block>
pub fn as_error(&self) -> Option<&String>
Sourceimpl StatementKind
impl StatementKind
pub fn as_assign_mut(&mut self) -> Option<(&mut Place, &mut Rvalue)>
pub fn as_set_discriminant_mut( &mut self, ) -> Option<(&mut Place, &mut VariantId)>
pub fn as_storage_live_mut(&mut self) -> Option<&mut LocalId>
pub fn as_storage_dead_mut(&mut self) -> Option<&mut LocalId>
pub fn as_place_mention_mut(&mut self) -> Option<&mut Place>
pub fn as_borrowck_mut(&mut self) -> Option<&mut BorrowckStatement>
pub fn as_drop_mut( &mut self, ) -> Option<(&mut Place, &mut FnPtr, &mut DropKind, &mut Block)>
pub fn as_assert_mut( &mut self, ) -> Option<(&mut Assert, &mut AbortKind, &mut Block)>
pub fn as_inline_asm_mut( &mut self, ) -> Option<(&mut String, &mut Vec<Block>, &mut Block)>
pub fn as_call_mut(&mut self) -> Option<(&mut Call, &mut Block)>
pub fn as_abort_mut(&mut self) -> Option<&mut AbortKind>
pub fn as_return_mut(&mut self) -> Option<()>
pub fn as_unwind_resume_mut(&mut self) -> Option<()>
pub fn as_break_mut(&mut self) -> Option<&mut usize>
pub fn as_continue_mut(&mut self) -> Option<&mut usize>
pub fn as_nop_mut(&mut self) -> Option<()>
pub fn as_switch_mut(&mut self) -> Option<&mut Switch>
pub fn as_loop_mut(&mut self) -> Option<&mut Block>
pub fn as_error_mut(&mut self) -> Option<&mut String>